@@ -452,6 +452,35 @@ def default_dbpn(
     freeze = False,
     verbose=False
  ):
+    """
+    Constructs a DBPN model based on input parameters, and can optionally
+    use external models for intensity or segmentation processing.
+
+    Args:
+        strider (list): List of strides, length must match `dimensionality`.
+        dimensionality (int): Number of dimensions (2 or 3). Default is 3.
+        nfilt (int): Number of base filters. Default is 64.
+        nff (int): Number of feature filters. Default is 256.
+        convn (int): Convolution kernel size. Default is 6.
+        lastconv (int): Size of the last convolution. Default is 3.
+        nbp (int): Number of back projection stages. Default is 7.
+        nChannelsIn (int): Number of input channels. Default is 1.
+        nChannelsOut (int): Number of output channels. Default is 1.
+        option (str): Model size option ('tiny', 'small', 'medium', 'large'). Default is None.
+        intensity_model (tf.keras.Model): Optional external intensity model.
+        segmentation_model (tf.keras.Model): Optional external segmentation model.
+        sigmoid_second_channel (bool): If True, applies sigmoid to second channel in output.
+        clone_intensity_to_segmentation (bool): If True, clones intensity model weights to segmentation.
+        pro_seg (int): If greater than 0, adds a segmentation arm.
+        freeze (bool): If True, freezes the layers of the intensity/segmentation models.
+        verbose (bool): If True, prints detailed logs.
+
+    Returns:
+        Model: A Keras model based on the specified configuration.
+
+    Raises:
+        Exception: If `len(strider)` is not equal to `dimensionality`.
+    """
     if option == 'tiny':
         nfilt=32
         nff = 64
@@ -1258,6 +1287,7 @@ def inference(
     segmentation=None,
     target_range=[1,0],
     poly_order='hist',
+    dilation_amount=0,
     verbose=False):
     if segmentation is None:
         pimg = ants.image_clone( image )
@@ -1300,7 +1330,7 @@ def inference(
                 mdl,
                 segmentation_numbers=mynp,
                 target_range=target_range,
-                dilation_amount=2,
+                dilation_amount=dilation_amount,
                 poly_order=poly_order,
                 max_lab_plus_one=True  )
         imgsr = temp['super_resolution' ]
@@ -1318,38 +1348,74 @@ def inference(
     return None
 
 
-def read_srmodel( srfilename ):
-    """
-    read a sr model with tf and return the model and the shape
 
-    Arguments
-    ---------
-    srfilename : string typically ending in .h5 extension
+def read_srmodel(srfilename, custom_objects=None):
+    """
+    Load a super-resolution model (h5, .keras, or SavedModel format),
+    and determine its upsampling factor.
+
+    Parameters
+    ----------
+    srfilename : str
+        Path to the model file (.h5, .keras, or a SavedModel folder).
+    custom_objects : dict, optional
+        Dictionary of custom objects used in the model (e.g. {'TFOpLambda': tf.keras.layers.Lambda(...)})
 
     Returns
     -------
-
-    tfmodel, upsampling_factor
-
-    upsampling_factor is [x_up, y_up, z_up, n_channels ] where these are integer values
+    model : tf.keras.Model
+        The loaded model.
+    upsampling_factor : list of int
+        List describing the upsampling factor:
+        - For 3D input: [x_up, y_up, z_up, channels]
+        - For 2D input: [x_up, y_up, channels]
 
     Example
     -------
-    >>> import siq
-    >>> mdl, upshape = siq.read_srmodel( "temp.h5" )
+    >>> mdl, up = read_srmodel("mymodel.keras")
+    >>> mdl, up = read_srmodel("my_weights.h5", custom_objects={"TFOpLambda": tf.keras.layers.Lambda(tf.identity)})
     """
-    srmdl = tf.keras.models.load_model( srfilename , compile=False )
-    chanindex = 3
-    if len( srmdl.input.shape ) == 5:
-        chanindex = 4
-    nchan = int(srmdl.input.shape[chanindex])
-    if len( srmdl.input.shape ) == 5:
-        outshap = srmdl( np.zeros([1,8,8,8,nchan]) ).shape
-        return srmdl, [int(outshap[1]/8),int(outshap[2]/8),int(outshap[3]/8),nchan]
-    else:
-        outshap = srmdl( np.zeros([1,8,8,nchan]) ).shape
-        return srmdl, [int(outshap[1]/8),int(outshap[2]/8),nchan]
 
+    # Expand path and detect format
+    srfilename = os.path.expanduser(srfilename)
+    ext = os.path.splitext(srfilename)[1].lower()
+
+    if os.path.isdir(srfilename):
+        # SavedModel directory
+        model = tf.keras.models.load_model(srfilename, custom_objects=custom_objects, compile=False)
+    elif ext in ['.h5', '.keras']:
+        model = tf.keras.models.load_model(srfilename, custom_objects=custom_objects, compile=False)
+    else:
+        raise ValueError(f"Unsupported model format: {ext}")
+
+    # Determine channel index
+    input_shape = model.input_shape
+    if isinstance(input_shape, list):
+        input_shape = input_shape[0]
+    chanindex = 3 if len(input_shape) == 4 else 4
+    nchan = int(input_shape[chanindex])
+
+    # Run dummy input to compute upsampling factor
+    try:
+        if len(input_shape) == 5:  # 3D
+            dummy_input = np.zeros([1, 8, 8, 8, nchan])
+        else:  # 2D
+            dummy_input = np.zeros([1, 8, 8, nchan])
+
+        # Handle named inputs if necessary
+        try:
+            output = model(dummy_input)
+        except Exception:
+            output = model({model.input_names[0]: dummy_input})
+
+        outshp = output.shape
+        if len(input_shape) == 5:
+            return model, [int(outshp[1]/8), int(outshp[2]/8), int(outshp[3]/8), nchan]
+        else:
+            return model, [int(outshp[1]/8), int(outshp[2]/8), nchan]
+
+    except Exception as e:
+        raise RuntimeError(f"Could not infer upsampling factor. Error: {e}")
 
 
 def simulate_image( shaper=[32,32,32], n_levels=10, multiply=False ):
@@ -1433,7 +1499,9 @@ def optimize_upsampling_shape( spacing, modality='T1', roundit=False, verbose=Fa
         tarshape = ["2","2","2"] # default
     return "x".join(tarshape)
 
-def compare_models( model_filenames, img, n_classes=3, identifier=None, noise_sd=0.1,verbose=False ):
+def compare_models( model_filenames, img, n_classes=3,
+    poly_order='hist',
+    identifier=None, noise_sd=0.1,verbose=False ):
     """
     generate a dataframe computing some basic intensity metrics PSNR and SSIM
 
@@ -1463,7 +1531,7 @@ def compare_models( model_filenames, img, n_classes=3, identifier=None, noise_sd
         if upshape[3] == 2:
             seghigh = ants.threshold_image( img,"Otsu",n_classes)
             seglow = ants.resample_image( seghigh, tarshape, use_voxels=False, interp_type=1 )
-            dimgup=inference( dimg, srmdl, segmentation = seglow, verbose=verbose )
+            dimgup=inference( dimg, srmdl, segmentation = seglow, poly_order=poly_order, verbose=verbose )
             dimgupseg = dimgup['super_resolution_segmentation']
             dimgup = dimgup['super_resolution']
             segblock = ants.resample_image_to_target( seghigh, dimgupseg, interp_type='nearestNeighbor'  )
@@ -1473,7 +1541,7 @@ def compare_models( model_filenames, img, n_classes=3, identifier=None, noise_sd
             dicenn = ants.label_overlap_measures(segblock, segimgnn)['MeanOverlap'][0]
             dicesr = ants.label_overlap_measures(segblock, dimgupseg)['MeanOverlap'][0]
         else:
-            dimgup=inference( dimg, srmdl, verbose=verbose )
+            dimgup=inference( dimg, srmdl, poly_order=poly_order, verbose=verbose )
         dimglin = ants.resample_image_to_target( dimg, dimgup, interp_type='linear' )
         imgblock = ants.resample_image_to_target( img, dimgup, interp_type='linear'  )
         dimgup[ imgblock == 0.0 ]=0.0
